import sys

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest

from mock import patch, call, Mock
from cctrl.error import InputErrorException
from cctrl.app import AppController
from cctrl.settings import Settings
from pycclib.cclib import GoneError


class AppControllerTestCase(unittest.TestCase):

    def test_get_size_from_memory_1gb(self):
        app = AppController(None, Settings())
        self.assertEqual(8, app._get_size_from_memory('1GB'))

    @patch('cctrl.app.sys')
    def test_get_size_from_memory_mb_rounded(self, _sys):
        app = AppController(None, Settings())
        self.assertEqual(6, app._get_size_from_memory('666MB'))
        self.assertEqual([
            call.stderr.write(
                'Memory size has to be a multiple of 128MB and has been ' +
                'rounded up to 768MB.'),
            call.stderr.write('\n')], _sys.mock_calls)

    def test_get_size_from_memory_nop_match(self):
        with self.assertRaises(InputErrorException) as ctx:
            AppController(None, Settings())._get_size_from_memory('0.7')
        self.assertEqual(
            '[ERROR] Memory size should be an integer between 128 and 1024 MB.',
            str(ctx.exception))

    def test_get_size_from_memory_unrecognized_unit(self):
        with self.assertRaises(InputErrorException) as ctx:
            AppController(None, Settings())._get_size_from_memory('4kb')
        self.assertEqual(
            '[ERROR] Memory size should be an integer between 128 and 1024 MB.',
            str(ctx.exception))

    @patch('cctrl.app.check_call')
    def test_push_with_ship(self, check_call):
        app = AppController(None, Settings())
        app.redeploy = Mock()
        app.log_from_now = Mock()
        app._get_or_create_deployment = Mock(
            return_value=({'branch': 'default', 'name': 'dep'}, 'name'))
        args = Mock()
        args.name = 'app/dep'
        args.deploy = False
        app.push(args)
        self.assertTrue(check_call.called)
        self.assertTrue(app.redeploy.called)
        self.assertTrue(app.log_from_now.called)

    @patch('cctrl.app.check_call')
    def test_push_with_deploy(self, check_call):
        app = AppController(None, Settings())
        app.redeploy = Mock()
        app._get_or_create_deployment = Mock(
            return_value=({'branch': 'default'}, 'name'))
        args = Mock()
        args.name = 'app/dep'
        args.ship = False
        app.push(args)
        self.assertTrue(check_call.called)
        self.assertTrue(app.redeploy.called)

    @patch('cctrl.app.check_call')
    def test_push_with_ship_and_deploy_error(self, check_call):
        app = AppController(None, Settings())
        app._get_or_create_deployment = Mock(
            return_value=({'branch': 'default', 'name': 'dep'}, 'name'))
        args = Mock()
        args.name = 'app/dep'
        with self.assertRaises(InputErrorException) as sd:
            app.push(args)
        self.assertEqual(
            '[ERROR] --ship and --push options cannot be used simultaneously.',
            str(sd.exception))

    def test_restart_worker_with_wrk(self):
        app = AppController(None, Settings())
        app._restartWorker = Mock()
        app._restartWorkers = Mock()
        app.api = Mock()
        app.api.read_worker.return_value = {'wrk_id': 'wrk1', 'command': 'command', 'params': 'params', 'size': 'size'}
        args = Mock()
        args.name = 'app/dep'
        args.wrk_id = 'wrk1'
        args.all = False
        app.restartWorker(args)
        self.assertTrue(app._restartWorker.called)
        self.assertFalse(app._restartWorkers.called)

    def test_restart_worker_with_all(self):
        app = AppController(None, Settings())
        app._restartWorker = Mock()
        app._restartWorkers = Mock()
        app.api = Mock()
        app.api.read_worker.return_value = {'wrk_id': 'wrk1', 'command': 'command', 'params': 'params', 'size': 'size'}
        args = Mock()
        args.name = 'app/dep'
        args.wrk_id = False
        args.all = True
        app.restartWorker(args)
        self.assertTrue(app._restartWorkers.called)

    def test_restart_worker_gone_error(self):
        app = AppController(None, Settings())
        app._restartWorker = Mock()
        app._restartWorkers = Mock()
        app.api = Mock()
        app.api.read_worker.side_effect = GoneError
        args = Mock()
        args.name = 'app/dep'
        args.wrk_id = 'wrkgone'
        args.all = False
        self.assertRaises(InputErrorException, app.restartWorker, args)
        self.assertFalse(app._restartWorker.called)
        self.assertFalse(app._restartWorkers.called)

    def test__restart_workers(self):
        app = AppController(None, Settings())
        app.api = Mock()
        app.api.read_workers.return_value = [{'wrk_id': 'wrk1'}]
        app.api.read_worker.return_value = {'wrk_id': 'wrk1', 'command': 'command', 'params': 'params', 'size': 8}

        app._restartWorkers('app', 'dep')

        self.assertTrue(app.api.delete_worker.called)
        self.assertTrue(app.api.create_worker.called)

    def test__restart_worker(self):
        app = AppController(None, Settings())
        app.api = Mock()
        app._restartWorker('app_name', 'deployment_name', 'wrk_id', 'command', 'params', 'size')
        self.assertTrue(app.api.delete_worker.called)
        self.assertTrue(app.api.create_worker.called)

    @patch('cctrl.app.time')
    def test_deploy_restart_workers(self, time):
        app = AppController(None, Settings())
        app.api = Mock()
        app.api.read_deployment.return_value = {'state': 'deployed'}
        app._restartWorkers = Mock()
        args = Mock()
        args.name = 'app/dep'
        args.memory = False
        app.deploy(args)
        self.assertTrue(app._restartWorkers.called)
        self.assertEqual(call('app', 'dep'), app.api.read_deployment.call_args_list[0])

    @patch('cctrl.app.time')
    def test_deploy_restart_workers_no_dep_name(self, time):
        app = AppController(None, Settings())
        app.api = Mock()
        app.api.read_deployment.return_value = {'state': 'deployed'}
        app._restartWorkers = Mock()
        args = Mock()
        args.name = 'app'
        args.memory = False
        app.deploy(args)
        self.assertTrue(app._restartWorkers.called)
        self.assertEqual(call('app', 'default'), app.api.read_deployment.call_args_list[0])



import os
import binascii
import sys

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest

from mock import patch, call, Mock, DEFAULT
from cctrl.keyhelpers import get_public_key_fingerprint, sign_token
import struct


@patch.multiple(
    '__builtin__',
    open=DEFAULT
)
@patch.multiple(
    'cctrl.keyhelpers',
    os=DEFAULT,
    get_default_ssh_key_path=Mock(return_value=os.path.expanduser('~') + '/.ssh/id_rsa.pub'),
    RSAKey=DEFAULT,
    paramiko=DEFAULT
)
class KeyHelpersTestCase(unittest.TestCase):
    TEST_PUB_KEY = '''ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQCpnGl7MiSArN350FHyQ+ZdgXlvMoCjPbghWStCnETpMimWe6VBNn4cmdfqc/wIJIWKW0WkndUAWkCaTG/1BAtDtxCPnrncwAqJxGsIsP14GPcsRMEfkYdStTQDyrAw3r1lFj8sxAvhtbxIzyfADrIbWh9Vu2RSZ1/oeKxncD6bNN7UPLmVd83hINjZZx/2AtbC19aVJ9ZV4QvsUUjYZVgPVMtu//8PtoSVOj+6GXZFzT1b85XIWxm7fi5iwA9p4Qbki7HcdVsfHpuJgM/bWyLBZ0bQQaY0UCsBEwD5Lq9wMlHYNna7POSvFPjZXpr/gzoQU7AFL6hjIRD7vVyNpn07 mw@cloudcontrol.de'''
    TEST_FINGERPRINT = '4a:4c:5b:3e:47:21:d5:7f:c9:8c:d9:2e:a7:22:73:65'

    def test_get_public_key_fingerprint(self, open, **kwargs):
        open.return_value.read.return_value = self.TEST_PUB_KEY
        self.assertEqual(self.TEST_FINGERPRINT, get_public_key_fingerprint('path'))
        self.assertEqual([call('path'), call().read()], open.mock_calls)

    def test_get_public_key_fingerprint_use_default_path(self, open, **kwargs):
        open.return_value.read.return_value = self.TEST_PUB_KEY
        self.assertEqual(self.TEST_FINGERPRINT, get_public_key_fingerprint(None))
        self.assertEqual([call(os.path.expanduser('~') + '/.ssh/id_rsa.pub'), call().read()], open.mock_calls)

    def test_get_public_key_fingerprint_key_not_found(self, open, **kwargs):
        open.side_effect = IOError('File not found')
        self.assertEqual(None, get_public_key_fingerprint('path'))
        self.assertEqual([call('path')], open.mock_calls)

    @patch('cctrl.keyhelpers.get_key_from_agent', Mock(return_value=None))
    def test_sign_token_with_private_key(self, os, RSAKey, **kwargs):
        os.path.exist.return_value = True

        private_key = Mock()
        prefix = 'ssh-rsa'
        content = "BBBBBB"
        private_key.sign_ssh_data.return_value = struct.pack('>I', len(prefix)) + prefix + struct.pack('>I', len(content)) + content

        RSAKey.from_private_key_file.return_value = private_key

        result = sign_token('/path/to/pkey.pub', 'fingerprint', 'somedata')
        self.assertEqual(content.encode('base64').strip(), result)

    def test_sign_token_with_agent(self, os, paramiko, **kwargs):
        os.path.exist.return_value = False

        private_key = Mock()
        private_key.get_fingerprint.return_value = binascii.a2b_hex('4a:4c:5b:3e:47:21:d5:7f:c9:8c:d9:2e:a7:22:73:65'.replace(':', ''))
        prefix = 'ssh-rsa'
        content = "BBBBBB"
        private_key.sign_ssh_data.return_value = struct.pack('>I', len(prefix)) + prefix + struct.pack('>I', len(content)) + content

        agent = paramiko.agent.Agent.return_value
        agent.get_keys.return_value = [private_key]

        result = sign_token(None, '4a:4c:5b:3e:47:21:d5:7f:c9:8c:d9:2e:a7:22:73:65', 'somedata')
        self.assertEqual(content.encode('base64').strip(), result)



import sys

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest

from mock import call, patch, Mock, MagicMock

from cctrl.user import UserController
from cctrl.error import InputErrorException
from cctrl.settings import Settings


@patch('cctrl.user.sys.stderr', MagicMock())
@patch('cctrl.user.readContentOf', MagicMock())
@patch('cctrl.user.get_default_ssh_key_path', Mock(return_value='/default/path/to/key'))
@patch('cctrl.user.create_new_default_ssh_keys')
@patch('cctrl.user.set_user_config')
class UserControllerTestCase(unittest.TestCase):
    @patch('cctrl.user.is_key_valid')
    @patch('cctrl.user.get_user_config')
    def test_setup(self,
                   _get_user_config,
                   _is_key_valid,
                   _set_user_config, *args):
        user = UserController(Mock(), Settings())
        user.api.read_users.return_value = [{'username': 'foo'}]
        args = Mock(email='info@example.org',
                    ssh_key_path='/path/to/key',
                    ssh_auth='no')

        _is_key_valid.return_value = True
        _get_user_config.return_value = {}

        user.setup(args)

        self.assertTrue(user.api.read_users.called)
        self.assertTrue(user.api.create_user_key.called)
        self.assertEqual(2, len(_set_user_config.mock_calls))
        self.assertEqual(call(user.settings, email='info@example.org'),
                         _set_user_config.mock_calls[0])
        self.assertEqual(call(user.settings,
                              ssh_auth=False,
                              ssh_path='/path/to/key'),
                         _set_user_config.mock_calls[1])

    @patch('cctrl.user.os.path.abspath', MagicMock())
    @patch('cctrl.user.is_key_valid')
    def test_setup_wrong_public_key(self, _is_key_valid, *args):
        user = UserController(Mock(), Settings())
        args = Mock(email='test@example.org',
                    ssh_key_path='not-existing',
                    ssh_auth='yes')

        _is_key_valid.return_value = False

        with self.assertRaises(InputErrorException) as iep:
            user.setup(args)

        self.assertEqual(
            '[ERROR] Public Key not found or invalid.',
            str(iep.exception))
        self.assertFalse(user.api.read_users.called)
        self.assertFalse(user.api.create_user_key.called)

    @patch('cctrl.user.is_key_valid')
    @patch('cctrl.user.get_user_config')
    def test_setup_non_existing_default_public_key(self,
                                                   _get_user_config,
                                                   _is_key_valid,
                                                   _set_user_config,
                                                   _create_default_ssh):
        user = UserController(Mock(), Settings())
        user.api.read_users.return_value = [{'username': 'foo'}]
        args = Mock(email='info@example.org',
                    ssh_key_path='/default/path/to/key',
                    ssh_auth='yes')

        _is_key_valid.return_value = False
        _get_user_config.return_value = {}

        user.setup(args)

        self.assertTrue(user.api.read_users.called)
        self.assertTrue(user.api.create_user_key.called)
        self.assertTrue(_create_default_ssh.called)
        self.assertEqual(2, len(_set_user_config.mock_calls))
        self.assertEqual(call(user.settings, email='info@example.org'),
                         _set_user_config.mock_calls[0])
        self.assertEqual(call(user.settings,
                              ssh_auth=True,
                              ssh_path='/default/path/to/key'),
                         _set_user_config.mock_calls[1])

    @patch('cctrl.user.is_key_valid')
    @patch('cctrl.user.get_user_config')
    def test_setup_no_args_provided(self,
                                    _get_user_config,
                                    _is_key_valid,
                                    _set_user_config, *args):
        user = UserController(Mock(), Settings())
        user.api.read_users.return_value = [{'username': 'foo'}]
        args = Mock(email=None,
                    ssh_key_path=None,
                    ssh_auth='yes')

        _get_user_config.return_value = {}
        _is_key_valid.return_value = True

        user.setup(args)

        self.assertTrue(user.api.read_users.called)
        self.assertTrue(user.api.create_user_key.called)
        self.assertEqual(1, len(_set_user_config.mock_calls))
        _set_user_config.assert_called_once_with(user.settings,
                                                 ssh_auth=True,
                                                 ssh_path='/default/path/to/key')

    @patch('cctrl.user.is_key_valid')
    @patch('cctrl.user.get_user_config')
    def test_setup_only_email_provided(self,
                                       _get_user_config,
                                       _is_key_valid,
                                       _set_user_config, *args):
        user = UserController(Mock(), Settings())
        user.api.read_users.return_value = [{'username': 'foo'}]
        args = Mock(email='info@example.org',
                    ssh_key_path=None,
                    ssh_auth=None)

        _get_user_config.return_value = {'ssh_auth': True,
                                         'ssh_path': '/path/to/key'}
        _is_key_valid.return_value = True

        user.setup(args)

        self.assertTrue(user.api.read_users.called)
        self.assertTrue(user.api.create_user_key.called)
        self.assertEqual(2, len(_set_user_config.mock_calls))
        self.assertEqual(call(user.settings, email='info@example.org'),
                         _set_user_config.mock_calls[0])
        self.assertEqual(call(user.settings,
                              ssh_auth=True,
                              ssh_path='/path/to/key'),
                         _set_user_config.mock_calls[1])



# -*- coding: utf-8 -*-
"""
    Copyright 2010 cloudControl UG (haftungsbeschraenkt)

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
import os
import sys

from __builtin__ import open, raw_input, range
from exceptions import ImportError, ValueError

import ConfigParser
from getpass import getpass
from cctrl.oshelpers import recode_input

try:
    import json
except ImportError:
    import simplejson as json

from pycclib import cclib

from cctrl.keyhelpers import get_default_ssh_key_path, \
    get_public_key_fingerprint, sign_token
from cctrl.error import SignatureException, \
    PublicKeyException, messages, PasswordsDontMatchException


def create_config_dir(settings):
    if os.path.isdir(settings.home_path):
        return
    elif os.path.isfile(settings.home_path):
        print >> sys.stderr, 'Error: ' + settings.home_path + ' is a file, not a directory.'
        sys.exit(1)
    else:
        os.mkdir(settings.home_path)


def create_token(api, settings, email, password):
    while True:
        try:
            if get_user_config(settings).get('ssh_auth') and password is None:
                key_path = get_user_config(settings).get('ssh_path',
                                                         get_default_ssh_key_path())
                fingerprint = get_public_key_fingerprint(key_path)
                if not fingerprint:
                    raise PublicKeyException('WrongPublicKey')

                ssh_token = api.create_ssh_token()
                signature = sign_token(key_path, fingerprint, ssh_token)
                return api.create_token_ssh_auth(email,
                                                 ssh_token,
                                                 signature,
                                                 fingerprint)
            else:
                return api.create_token_basic_auth(email, password)

        except (cclib.APIException, SignatureException, PublicKeyException) as e:
            if password is not None:
                sys.exit(messages['NotAuthorized'])

            print >> sys.stderr, str(e) + " " + messages['NotAuthorizedPublicKey']
            password = get_password_env(settings)
            if not password:
                password = get_password()


def update_tokenfile(api, settings):
    """
        Because it is a real pain we don't want to ask developers for their
        username and password every time they call a method.

        Therefore we authenticate users via token for each request and only
        require email and password for a new token.
        A token is valid for a given period of time. Each successful API
        request resets the expiration time.
    """
    if api.check_token():
        write_tokenfile(api, settings)
        return True
    return False


def read_tokenfile(settings):
    """
        Read the token from the token_path specified in
        cctrl.settings
    """
    token = None
    if os.path.exists(settings.token_path):
        token_file = open(settings.token_path, "r")
        try:
            token = json.load(token_file)
        except ValueError:
            token = None
        token_file.close()
    return token


def write_tokenfile(api, settings):
    """
        This method checks, if the .cloudControl directory inside the
        users home exists or is a file. If not, we create it and then
        write the token file.
    """
    create_config_dir(settings)

    token_file = open(settings.token_path, "w")
    json.dump(api.get_token(), token_file)
    token_file.close()
    return True


def delete_tokenfile(settings):
    """
        We delete the tokenfile if we don't have a valid token to save.
    """
    if os.path.exists(settings.token_path):
        os.remove(settings.token_path)
        return True
    return False


def set_user_config(settings, email=None, ssh_auth=None, ssh_path=None):
    create_config_dir(settings)
    config = ConfigParser.ConfigParser()
    config.read(settings.config_path)

    if not config.has_section('user'):
        config.add_section('user')

    if email:
        config.set('user', 'email', email)

    if ssh_auth is not None:
        config.set('user', 'ssh_auth', ssh_auth)

    if ssh_path:
        config.set('user', 'ssh_path', ssh_path)

    with open(settings.config_path, 'w') as user_config:
        config.write(user_config)


def get_user_config(settings):
    config = ConfigParser.ConfigParser()
    config.read(settings.config_path)
    if config.has_section('user'):
        cf = dict(config.items('user'))
        if 'ssh_auth' in cf:
            cf['ssh_auth'] = config.getboolean('user', 'ssh_auth')

        return cf

    return {}


def get_email_and_password(settings):
    email = get_email_env(settings) or \
        get_user_config(settings).get('email') or \
        get_email(settings)

    if get_user_config(settings).get('ssh_auth'):
        return email, None

    password = get_password_env(settings)
    if password is None:
        password = get_password()

    return email, password


def get_email(settings):
    sys.stderr.write(settings.login_name)
    sys.stderr.flush()

    email = raw_input()
    set_user_config(settings, email=email)
    return email


def get_email_env(settings):
    try:
        email = os.getenv(settings.login_creds['email'])
    except KeyError:
        return None

    return email


def get_password(create=False):
    password = None
    for i in range(3):
        password = recode_input(getpass('Password: '))
        if create:
            password2 = recode_input(getpass('Password (again): '))
            if password != password2:
                print >> sys.stderr, messages['PasswordsDontMatch']
                if i == 2:
                    raise PasswordsDontMatchException()
            else:
                break
        else:
            break

    return password


def get_password_env(settings):
    try:
        password = os.getenv(settings.login_creds['pwd'])
    except KeyError:
        return None

    return password


def get_credentials(settings, create=False):
    """
        We use this to ask the user for his credentials in case we have no
        valid token.
        If create is true, the user is asked twice for the password,
        to make sure, that no typing error occurred. This is done three times
        after that a PasswordsDontMatchException is thrown.
    """

    email = get_email(settings)

    password = get_password(create)

    return email, password
